
#include "stdafx.h"
#include "MCal_Func.h"
#include "visa.h"


/////////////////////////////////////////////////////////////////////////////
#define	STR_POWER_SENSOR			"HP8482A"


static const double dFreqTab[] = {
	100E3,	1E6,	5E6,	10E6,	15E6,
	20E6,	21E6,	22E6,	23E6,	24E6,
	25E6,	26E6,	27E6,	28E6,	29E6,
	32E6,	37E6,	38E6,	39E6,	40E6,
	41E6,	42E6,	43E6,	48E6,	50E6,
};

const char *strChange_Frequency =
{
	"Connect AWG OUTPUT to Channel 1 of 53132A !"
};

const char *strChange_ADC =
{
	"Connect AWG OUTPUT to AWG \"Modulation In\" on Rear Panel and 3458A INPUT !"
};

const char *strChange_Volt =
{
	"Connect AWG OUTPUT to 3458A INPUT !"
};

const char *strChange_Flatness =
{
	"Connect AWG OUTPUT to POWER SENSOR !"
};

/////////////////////////////////////////////////////////////////////////////
void MySleep(DWORD dwMS)
{
	MSG msg;
	DWORD dwCount = GetTickCount() + dwMS;
	while(GetTickCount() < dwCount){
		while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)){
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		Sleep(1);
	}
}

/////////////////////////////////////////////////////////////////////////////
static void Init_PowerMeter(ViSession vi)
{
	viClear(vi);
//	viPrintf(vi, "*RST\n");
//	MySleep(1000);
	viPrintf(vi, "*CLS\n");
	viPrintf(vi, "CORR:CSET1 \"%s\"\n", STR_POWER_SENSOR);	// Select Power Sensor
	viPrintf(vi, "DISP:RES 3\n");	// Resolution x.xx 
	viPrintf(vi, "FREQ 100E3\n");
}

static void Init_MultiMeter_DC(ViSession vi)
{
	//	3458A
	viPrintf(vi, "PRESET NORM\n");
	viPrintf(vi, "FUNC DCV\n");
	viPrintf(vi, "DELAY 1\n");
	viPrintf(vi, "END ALWAYS\n");
}

static void Init_MultiMeter_AC(ViSession vi)
{
	//	3458A
	viPrintf(vi, "PRESET NORM\n");
	viPrintf(vi, "FUNC ACV\n");
	viPrintf(vi, "DELAY 1\n");
	viPrintf(vi, "END ALWAYS\n");
}

static void Init_Counter(ViSession vi)
{
	//	53132A
	viClear(vi);
	viPrintf(vi, "*RST\n");
	viPrintf(vi, "CONF:FREQ\n");
	viPrintf(vi, "INP:IMP 50\n");
	viPrintf(vi, "INP:COUP DC\n");
	viPrintf(vi, "FREQ:ARM:START:SOUR IMM\n");
	viPrintf(vi, "FREQ:ARM:STOP:SOUR TIM\n");
	viPrintf(vi, "FREQ:ARM:STOP:TIM 1\n");	// Set Gate Time = 1 Sec
	viPrintf(vi, "INIT:CONT ON\n");
	MySleep(2500);	// Wait counter ready to run
}

/////////////////////////////////////////////////////////////////////////////
static ViStatus Meas_PowerMeter_Pwr(ViSession vi, double *value, double dParm)
{
	*value = 0.0;

	viPrintf(vi, "FREQ %f\n", dParm);
	MySleep(500);
	return viQueryf(vi, "MEAS?\n", "%le", value);	// dBm
}

static ViStatus Meas_MultiMeter_Volt(ViSession vi, double *value, double dParm)
{
	*value = 0.0;

	MySleep(1);	// Transmission system message
	return viScanf(vi, "%le", value);
}

static ViStatus Meas_Counter_Freq(ViSession vi, double *value, double dParm)
{
	*value = 0.0;

	MySleep(3000);
	return viQueryf(vi, "SENS:DATA?\n", "%le", value);
}

/////////////////////////////////////////////////////////////////////////////
int nCurSetup, nPreSetup;

enum SetupNum
{
	SN_SELF_TEST	= 1,
	SN_FREQ			= 2,
	SN_ADC			= 6,
	SN_SELF_CAL		= 7,
	SN_AC_AMP		= 8,

	SN_FLTNS_0DBM	= (24+AC_AMP_EXT_COUNT),
	SN_FLTNS_10DBM	= (49+AC_AMP_EXT_COUNT),
	SN_FLTNS_20DBM	= (74+AC_AMP_EXT_COUNT),

	SN_MAX			= (98+AC_AMP_EXT_COUNT)
};

tagCAL_SETUP Cal_Setup[] =
{
	// 1
	{ SN_SELF_TEST,		"Self-Test",			NULL,					-1,	NULL,	NULL,	NULL	},

	// 2 - 5
	{ SN_FREQ,			"Frequency",			strChange_Frequency,	4,	&Init_Counter,			&Meas_Counter_Freq,		NULL	},

	// 6
	{ SN_ADC,			"ADC",					strChange_ADC,			1,	&Init_MultiMeter_DC,	&Meas_MultiMeter_Volt,	NULL	},

	// 7
	{ SN_SELF_CAL,		"Self-Cal",				NULL,					-1,	NULL,	NULL,	NULL	},

	// 8 - 24
	{ SN_AC_AMP,		"AC Amplitude",			strChange_Volt,			(16+AC_AMP_EXT_COUNT),	&Init_MultiMeter_DC,	&Meas_MultiMeter_Volt,	NULL	},

	// 24 - 48
	{ SN_FLTNS_0DBM,	"Flatness 0dBm",		strChange_Flatness,		25,	&Init_PowerMeter,		&Meas_PowerMeter_Pwr,	(double *)dFreqTab	},

	// 49 - 73
	{ SN_FLTNS_10DBM,	"Flatness 10dBm",		strChange_Flatness,		25,	&Init_PowerMeter,		&Meas_PowerMeter_Pwr,	(double *)dFreqTab	},

	// 74 - 98
	{ SN_FLTNS_20DBM,	"Flatness 20dBm",		strChange_Flatness,		25,	&Init_PowerMeter,		&Meas_PowerMeter_Pwr,	(double *)dFreqTab	},

	{ -1	}
};

/////////////////////////////////////////////////////////////////////////////
CManualAdj::CManualAdj(HWND &m_hWnd)
{
	m_pExtWnd = &m_hWnd;
	if(m_pExtWnd == NULL)
		AfxMessageBox("The program will be crash !\n\nBecause m_pExtWnd can't be NULL !", MB_ICONSTOP);

	nPreSetup = nCurSetup = CS_SELF_TEST;

	viMeter = viAWG = 0;
	viOpenDefaultRM(&defRM);
}

CManualAdj::~CManualAdj()
{
	m_pExtWnd = NULL;
	nPreSetup = nCurSetup = CS_SELF_TEST;

	CloseINSTR_Meter();
	CloseINSTR_AWG();
	viClose(defRM);
}

BOOL CManualAdj::OpenINSTR_Meter(LPSTR addr)
{
	if(viMeter != 0)
		CloseINSTR_Meter();

	status = viOpen(defRM, addr, VI_NULL, 3000, &viMeter);

	if(status != VI_SUCCESS)
		MessageBox(*m_pExtWnd, addr, "Open Meter connection fail !", MB_ICONSTOP);

	return (status != VI_SUCCESS);
}

BOOL CManualAdj::OpenINSTR_AWG(LPSTR addr)
{
	if(viAWG != 0)
		CloseINSTR_AWG();

	status = viOpen(defRM, addr, VI_NULL, 3000, &viAWG);

	if(status != VI_SUCCESS)
		MessageBox(*m_pExtWnd, addr, "Open AWG connection fail !", MB_ICONSTOP);

	return (status != VI_SUCCESS);
}

void CManualAdj::CloseINSTR_Meter()
{
	viClose(viMeter);
	viMeter = 0;
}

void CManualAdj::CloseINSTR_AWG()
{
	viClose(viAWG);
	viAWG = 0;
}

BOOL CManualAdj::IsErr()
{
	char sRevBuf[16] = {0};

	int flag,
		mask = (0x01 << 2);	// Error Queue Flag

	status = viQueryf(viAWG, "*STB?\n", "%t", sRevBuf);
	flag = mask & atoi(sRevBuf);

	return ((status == VI_SUCCESS) && (BOOL)flag);
}

void CManualAdj::GetCurSetupTitle(LPSTR lpText)
{
	strcpy(lpText, Cal_Setup[nCurSetup].sTitle);
}

void CManualAdj::GetCalErrMsg(CString &csErr)
{
	char sRevBuf[1024] = {0};

	csErr.Empty();
	do
	{
		status = viQueryf(viAWG, "SYST:ERR?\n", "%t", sRevBuf);
		if(!strcmp("+0,\"No error\"\n", sRevBuf)) break;

		csErr += sRevBuf;
	}
	while(status == VI_SUCCESS);
}

BOOL CManualAdj::IsErrorAndDisp()
{
	BOOL bChk = IsErr();

	if(bChk)
	{
		char sTital[128], sBuf[128];
		int nStep;
		CString csErr;

		GetCurSetupTitle(sTital);
		csErr.Format("Error ocurr at Calibration \"%s\" !", sTital);
		strcpy(sTital, csErr.GetBuffer(0));

		nStep = 0;
		viQueryf(viAWG, "CAL:SET?\n", "%d", &nStep);
		sprintf(sBuf, "Stop at \"Setup %d\"\n\n", nStep);

		GetCalErrMsg(csErr);
		csErr = sBuf + csErr;
		SetForegroundWindow(*m_pExtWnd);
		MessageBox(*m_pExtWnd, csErr, sTital, MB_ICONSTOP);
	}

	return bChk;
}

BOOL CManualAdj::IsSecuredAWG()
{
	int bChk=0;

	status = viQueryf(viAWG, "CAL:SEC:STAT?\n", "%d", &bChk);
	return ((status == VI_SUCCESS) && bChk);
}

void CManualAdj::SetCalSetup(enum CalSetup step, BOOL bInit)
{
	nCurSetup = step;

	if(bInit)
		InitEquipState();
}

void CManualAdj::SetSecureAWG(LPCSTR psswrd)
{
	viPrintf(viAWG, "CAL:SEC:CODE %s\n", psswrd);
	viPrintf(viAWG, "CAL:SEC:STAT ON,%s\n", psswrd);
}

tagCAL_SETUP *CManualAdj::GetCurCalSetup() const
{
	return &Cal_Setup[nCurSetup];
}

void CManualAdj::InitEquipState()
{
	if(GetCurCalSetup() == NULL) return;

	// AWG initialization
	viPrintf(viAWG, "CAL:SET %d\n", GetCurCalSetup()->nStep);
	IsErrorAndDisp();

	// Meter initialization
	if(GetCurCalSetup()->MeterInit != NULL)
		GetCurCalSetup()->MeterInit(viMeter);
}

void CManualAdj::UnSecuredAWG(LPCSTR psswrd)
{
	viPrintf(viAWG, "CAL:SEC:STAT OFF,%s\n", psswrd);
}

BOOL CManualAdj::CalExecute()
{
	tagCAL_SETUP *lpCal = GetCurCalSetup();
	char sRevBuf[1024];
	double dMeas = 0.0;
	int j,
		nStep = lpCal->nStep;

	CString csCurStep;


	if(lpCal == NULL) return FALSE;

	// Show change cable TIP
	if(lpCal->sChangeTip != NULL)
	{
		if(	(nCurSetup == CS_FLTNS_0DBM		||
			 nCurSetup == CS_FLTNS_10DBM	||
			 nCurSetup == CS_FLTNS_20DBM)	&&
			(nPreSetup == CS_FLTNS_0DBM		||
			 nPreSetup == CS_FLTNS_10DBM	||
			 nPreSetup == CS_FLTNS_20DBM)
		)
		{
			// Do nothing
		}
		else
		{
			SetForegroundWindow(*m_pExtWnd);
			if(IDYES != AfxMessageBox(lpCal->sChangeTip, MB_ICONWARNING | MB_YESNO))
				return FALSE;
		}
	}

	// Configure INSTR
	viPrintf(viAWG, "*CLS\n");
	switch(nStep)
	{
		case SN_SELF_TEST:
			viSetAttribute(viAWG, VI_ATTR_TMO_VALUE, 10*1000);	// TMO 10 sec
			break;
		case SN_SELF_CAL:
			viSetAttribute(viAWG, VI_ATTR_TMO_VALUE, 30*1000);	// TMO 30 sec
			break;

		default:
			viSetAttribute(viAWG, VI_ATTR_TMO_VALUE, 5*1000);	// TMO 5 sec
			break;
	}

	/*
	** Perform multi-setup and need to enter value of setup.
	** Such as Freq, ADC, AC Ampl, Flatness
	*/
	if(lpCal->MeterMeas != NULL)
	{
		if(lpCal->nTestCount > 0)	/* Loop Meas */
		{
			for(j=0; j < lpCal->nTestCount; j++)
			{
				// Next Setup
				if(j > 0)
					viPrintf(viAWG, "CAL:SET %d\n", ++nStep);

				// Get measurement value
				if(lpCal->MeterExtParm != NULL)
					status = lpCal->MeterMeas( viMeter, &dMeas, lpCal->MeterExtParm[j] );	// Multi-Calibration Step
				else
					status = lpCal->MeterMeas(viMeter, &dMeas);	// Single Calibration Step

				if(status != VI_SUCCESS)
				{
					SetForegroundWindow(*m_pExtWnd);
					csCurStep.Format("Stop at \"Setup %d\"", nStep);
					MessageBox(*m_pExtWnd, csCurStep, "NI-VISA communication error !", MB_ICONSTOP);
					return FALSE;
				}

				// Configure input value of unit
				switch(nCurSetup)
				{
					case CS_FLTNS_0DBM:
					case CS_FLTNS_10DBM:
					case CS_FLTNS_20DBM:
						viPrintf(viAWG, "CAL:MVAL %.3fdBm\n", dMeas);	// dBm
						break;

					default:
						viPrintf(viAWG, "CAL:MVAL %f\n", dMeas);
						break;
				}

				// Get AWG error flag
				if(IsErrorAndDisp())
					return FALSE;

				// Excute "CAL?"
				sRevBuf[0] = '\0';
				status = viQueryf(viAWG, "CAL?\n", "%t", sRevBuf);
				if(status == VI_SUCCESS)
				{
					if(atoi(sRevBuf) != 0)
						return !IsErrorAndDisp();
				}
				else
				{
					// If input value is in invalid range in current setup, the AWG won't respond.
					// Try to get error message from AWG
					if(!IsErrorAndDisp())
					{
						csCurStep.Format("Stop at \"Setup %d\"", nStep);
						MessageBox(*m_pExtWnd, csCurStep, "NI-VISA communication error !", MB_ICONSTOP);
					}
					return FALSE;
				}
			}

			nPreSetup = nCurSetup;
			return TRUE;
		}

	}		/* Loop Meas */

	/*
	** Perform single setup or needless to enter value of setup.
	** Such as Self-Test, Self-Cal
	*/
	// Excute "CAL?"
	sRevBuf[0] = '\0';
	status = viQueryf(viAWG, "CAL?\n", "%t", sRevBuf);
	if(status == VI_SUCCESS)
	{
		nPreSetup = nCurSetup;
		if(atoi(sRevBuf) == 0)
			return TRUE;
		else
			return !IsErrorAndDisp();
	}
	else
	{
		// If input value is in invalid range in current setup, the AWG won't respond.
		// Try to get error message from AWG
		if(!IsErrorAndDisp())
		{
			csCurStep.Format("Stop at \"Setup %d\"", nStep);
			MessageBox(*m_pExtWnd, csCurStep, "NI-VISA communication error !", MB_ICONSTOP);
		}
		return FALSE;
	}
}

void CManualAdj::CalPowerSensor()
{
	char RevBuf[128];
	double dSTB = 0;
	int Retry = 0;

	viPrintf(viMeter, "FREQ 50E6\n");
	viPrintf(viMeter, "CORR:CSET1 \"%s\"\n", STR_POWER_SENSOR);	// Select Power Sensor
	
	viPrintf(viMeter, "CAL:ZERO:AUTO ONCE\n");
	MySleep(7000);
	viQueryf(viMeter, "*OPC?\n", "%t", RevBuf);
	
	do
	{
		if(dSTB == 4){
			viQueryf(viMeter,"SYST:ERR?\n", "%t", RevBuf);
			MessageBox(	*m_pExtWnd,
						"Connect "STR_POWER_SENSOR" to E4418 POWER REF !!!",
						"ACTION",
						MB_ICONWARNING );
		}	
		viPrintf(viMeter,"CAL:AUTO ONCE\n");
		MySleep(3000);
		viQueryf(viMeter, "*OPC?\n", "%t", RevBuf);
		viQueryf(viMeter, "*STB?\n", "%t", RevBuf);
		dSTB = strtod(RevBuf, 0);
		Retry++;
	}while(dSTB && Retry < 3);

}